#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

#import pyjamendo
#import simplejson

import os
from time import time
try:
    from pysqlite2 import dbapi2 as sqlite3
except ImportError:
    import sqlite3

import functions

# Gettext - Übersetzung
functions.translation_gettext()
#def _(string):
#    return string

#
# Database class for general values
#
class DB_Settings():
    def __init__(self, pyjama):
        self.pyjama = pyjama
        self.db = os.path.join(self.pyjama.home, "content.db")

    ###################################################################
    #
    # Opening SQL connection
    #
    def open(self):
        self.conn = sqlite3.connect(self.db)

    ###################################################################
    #
    # Close SQL connection
    #
    def close(self):
        self.conn.close()

    ###################################################################
    #
    # A SQL query
    #
    def query(self, query):
        try:
            self.open()
            cur = self.conn.cursor()
            cur.execute(query)
            rows = cur.fetchall()
       
            self.close()
        
            return rows
        except Exception, inst:
            desc = "There was an error while querying the database"
            self.parent.Events.raise_event("error", inst, desc)
            return -1

#
# Database for jamendo's databasedumps
#
class DB():
    def __init__(self, par):
        self.parent = par
        self.home=functions.preparedirs()
        self.db = os.path.join(self.home, "pyjama.db")

        self.tracks = self.query("SELECT COUNT (*) FROM tracks WHERE 1")[0][0]
        self.albums = self.query("SELECT COUNT (*) FROM albums WHERE 1")[0][0]
        self.artists = self.query("SELECT COUNT (*) FROM artists WHERE 1")[0][0]
        
        self.results = {}
        #print "%i Tracks in Database" % self.query("SELECT COUNT (*) FROM tracks WHERE 1")[0][0]
        #print "%i Albums in Database" % self.query("SELECT COUNT (*) FROM albums WHERE 1")[0][0]
        #print "%i Artists in Database" % self.query("SELECT COUNT (*) FROM artists WHERE 1")[0][0]

    ###################################################################
    #
    # Opening SQL connection
    #
    def open(self):
        self.conn = sqlite3.connect(self.db)

    ###################################################################
    #
    # Close SQL connection
    #
    def close(self):
        self.conn.close()

    ###################################################################
    #
    # A SQL query
    #
    def query(self, query):
        try:
            self.open()
            cur = self.conn.cursor()
            cur.execute(query)
            rows = cur.fetchall()
       
            self.close()
        
            return rows
        except Exception, inst:
            desc = "There was an error while querying the database"
            self.parent.Events.raise_event("error", inst, desc)
            return -1

    ###################################################################
    #
    # get several informations for a spicific artist-id
    #
    def artistinfos(self, id, thread_id = 0):
        sql = """
            SELECT
                artists.name, artists.country, artists.image, artists.url, artists.id, albums.name, albums.id, artists.albumcount
            FROM
                artists, albums
            WHERE
                artists.id=%s and albums.artist_id=artists.id
            """ % str(id)
        ret = self.query(sql)
        counter = 0
        artist = {}
        for info in ret:
            artist[counter] = {}
            artist[counter]['artist_name'] = info[0]
            artist[counter]['artist_country'] = info[1]
            artist[counter]['artist_image'] = info[2]
            artist[counter]['artist_url'] = info[3]
            artist[counter]['artist_id'] = info[4]
            artist[counter]['album_name'] = info[5]
            artist[counter]['album_id'] = info[6]
            artist[counter]['artist_albumcount'] = info[7]
            counter += 1
        self.results[thread_id]=artist
        return artist

    ###################################################################
    #
    # get track-informations for several track-ids
    #
    def multiple_albumtracks(self, ids):
        where_clause = ") OR ("
        strings = []
        for id in ids:
            strings.append( "tracks.albumID='%s'" % id)
        where_clause = "(" + where_clause.join(strings) + ")"
        sql = """
            SELECT
                tracks.dispname, tracks.lengths, tracks.trackno, tracks.id, albums.id, albums.dispname, artists.id, artists.dispname, artists.name
            FROM
                albums, artists
            INNER JOIN tracks ON albums.artistID=tracks.albumID
            """# % where_clause #order by tracks.albumID
        print sql
        ret = self.query(sql)
        counter = 0
        tracks={}
        for track in ret:
            tracks[counter] = {}
            #tracks[counter]['uid'] = time()
            tracks[counter]['dispname'] = track[0]
            tracks[counter]['lengths'] = track[1]
            tracks[counter]['trackno'] = track[2]
            tracks[counter]['id'] = track[3]
            tracks[counter]['album_id'] = track[4]
            tracks[counter]['album_dispname'] = track[5]
            tracks[counter]['artist_id'] = track[6]
            tracks[counter]['artist_dispname'] = track[7]
            tracks[counter]['artist_name'] = track[8]
#            tracks[counter]['stream'] = "query" 
            tracks[counter]['stream'] = "http://api.jamendo.com/get2/stream/track/redirect/?id=%i&streamencoding=ogg2" % track[3]
            counter += 1
        return tracks

    def get_trackinfos2(self, trackid):
        sql = """
            SELECT
                tracks.name, tracks.duration, tracks.numalbum, tracks.license, tracks.album_id, tracks.artist_id, artists.name, albums.name, tracks.uid
            FROM
                tracks, artists, albums
            WHERE
                tracks.id=%s and albums.id=tracks.album_id and artists.id=tracks.artist_id
            """ % str(trackid)
        ret = self.query(sql)
        counter = 0
        tracks={}
        for track in ret:
            tracks[counter] = {}
            tracks[counter]['uid'] = track[8]
            tracks[counter]['id'] = trackid
            tracks[counter]['name'] = track[0]
            tracks[counter]['duration'] = track[1]
            tracks[counter]['numalbum'] = track[2]
            tracks[counter]['license'] = track[3]
            tracks[counter]['artist_id'] = track[5]
            tracks[counter]['artist_name'] = track[6]
            tracks[counter]['album_id'] = track[4]
            tracks[counter]['album_name'] = track[7]
#            tracks[counter]['stream'] = "query"
            tracks[counter]['stream'] = "http://api.jamendo.com/get2/stream/track/redirect/?id=%s&streamencoding=ogg2" % str(trackid)
            counter += 1
        return tracks

    ###################################################################
    #
    # get track-informations for a spicific track
    #
    def get_trackinfos(self, trackid, albumid, artistid):

        sql = """
            SELECT
                tracks.name, tracks.duration, tracks.numalbum, tracks.license, artists.name, albums.name
            FROM
                tracks, artists, albums
            WHERE
                tracks.id=%s and albums.id=%s and artists.id=%s
            """ % (str(trackid), str(albumid), str(artistid))
        ret = self.query(sql)
        counter = 0
        tracks={}
        for track in ret:
            tracks[counter] = {}
            #tracks[counter]['uid'] = time()
            tracks[counter]['id'] = trackid
            tracks[counter]['name'] = track[0]
            tracks[counter]['duration'] = track[1]
            tracks[counter]['numalbum'] = track[2]
            tracks[counter]['license'] = track[3]
            tracks[counter]['artist_id'] = artistid
            tracks[counter]['artist_name'] = track[4]
            tracks[counter]['album_id'] = albumid
            tracks[counter]['album_name'] = track[5]
#            tracks[counter]['stream'] = "query"
            tracks[counter]['stream'] = "http://api.jamendo.com/get2/stream/track/redirect/?id=%i&streamencoding=ogg2" % trackid
            counter += 1
        return tracks
        
        
#http://api.jamendo.com/get2/track_name+track_duration+track_url+license_url+album_id+album_name+artist_id+artist_idstr+artist_name/track/jsonpretty/album_artist/

    ###################################################################
    #
    # get all artist's tracks
    #
    def artisttracks(self, id):
        sql = """
            SELECT
                tracks.name, tracks.duration, tracks.numalbum, tracks.id, tracks.license, tracks.id3genre, albums.id, albums.name
            FROM
                tracks, albums
            WHERE
                tracks.album_id=albums.id and tracks.artist_id=%s and albums.artist_id=%s
            """ % (str(id), str(id))
        ret = self.query(sql)                
        counter = 0
        tracks={}
        for track in ret:
            tracks[counter] = {}
            #tracks[counter]['uid'] = time()
            tracks[counter]['name'] = track[0]
            tracks[counter]['duration'] = track[1]
            tracks[counter]['numalbum'] = track[2]
            tracks[counter]['id'] = track[3]
            tracks[counter]['license'] = track[4]
            tracks[counter]['id3genre'] = track[5]
            tracks[counter]['album_id'] = track[6]
            tracks[counter]['album_name'] = track[7]
#            tracks[counter]['artist_id'] = track[8]
#            tracks[counter]['artist_name'] = track[9]
#            tracks[counter]['stream'] = "query"
            tracks[counter]['stream'] = "http://api.jamendo.com/get2/stream/track/redirect/?id=%i&streamencoding=ogg2" % track[3]
            counter += 1
        return tracks

    ###################################################################
    #
    # get all tracks of an album
    #
    def albumtracks(self, id):

        sql = """
            SELECT
                tracks.name, tracks.duration, tracks.numalbum, tracks.id, tracks.license, tracks.id3genre, albums.id, albums.name, artists.id, artists.name
            FROM
                tracks, albums, artists
            WHERE
                tracks.album_id=%s and albums.id=%s and artists.id=albums.artist_id
            """ % (str(id), str(id))
        ret = self.query(sql)
        counter = 0
        tracks={}
        for track in ret:
            tracks[counter] = {}
            #tracks[counter]['uid'] = time()
            tracks[counter]['name'] = track[0]
            tracks[counter]['duration'] = track[1]
            tracks[counter]['numalbum'] = track[2]
            tracks[counter]['id'] = track[3]
            tracks[counter]['license'] = track[4]
            tracks[counter]['id3genre'] = track[5]
            tracks[counter]['album_id'] = track[6]
            tracks[counter]['album_name'] = track[7]
            tracks[counter]['artist_id'] = track[8]
            tracks[counter]['artist_name'] = track[9]
#            tracks[counter]['stream'] = "query"
            tracks[counter]['stream'] = "http://api.jamendo.com/get2/stream/track/redirect/?id=%i&streamencoding=ogg2" % track[3]
            counter += 1
        return tracks

    ###################################################################
    #
    # search for tags
    #
    def search_tag(self, string):
    
        sql = """
            SELECT
                albums.name, albums.id, artists.name, artists.id
            FROM
                albums, artists, tags
            WHERE
                tags.idstr='""" + string + """' and albums.id=tags.album_id LIMIT """ + str(self.parent.settings.get_value("PERFORMANCE", "db_search_limit"))
        ret = self.query(sql)
        tags = {}
        counter = 0
        for tag in ret:
            tags[counter] = {}
            tags[counter]['album_name'] = tag[0]
            tags[counter]['album_id'] = tag[1]
            tags[counter]['artist_name'] = tag[2]
            tags[counter]['artist_id'] = tag[3]
            counter += 1
        print tags
        return tags


    ###################################################################
    #
    # search for tracks
    #
    def search_track(self, string):
    
        sql = """
            SELECT
                tracks.name, tracks.id, tracks.numalbum, tracks.license, albums.id, albums.name, artists.id, artists.name
            FROM
                tracks, albums, artists
            WHERE
                tracks.name LIKE '%""" + string + """%' and tracks.album_id=albums.id and albums.artist_id=artists.id LIMIT """ + str(self.parent.settings.get_value("PERFORMANCE", "db_search_limit"))
        ret = self.query(sql)
        tracks = {}
        counter = 0
        for track in ret:
            tracks[counter] = {}
            tracks[counter]['track_name'] = track[0]
            tracks[counter]['track_id'] = track[1]
            tracks[counter]['track_numalbum'] = track[2]
            tracks[counter]['track_license'] = track[3]
            tracks[counter]['album_id'] = track[4]
            tracks[counter]['album_name'] = track[5]
            tracks[counter]['artist_id'] = track[6]
            tracks[counter]['artist_name'] = track[7]
            counter += 1
        return tracks

    ###################################################################
    #
    # search for artists
    #
    def search_artist(self, string):
        sql = """
            SELECT
                artists.name, artists.id
            FROM
                artists
            WHERE
                artists.name LIKE '%""" + string + """%' LIMIT """ + str(self.parent.settings.get_value("PERFORMANCE", "db_search_limit"))
        ret = self.query(sql)
        artists = {}
        counter = 0
        for artist in ret:
            artists[counter] = {}
            artists[counter]['artist_name'] = artist[0]
            artists[counter]['artist_id'] = artist[1]
#            artists[counter]['album_dispname'] = artist[2]
#            artists[counter]['album_id'] = artist[3]
            counter += 1
        return artists

    ###################################################################
    #
    # search for albums
    #        
    def search_album(self, string):
    
        sql = """
            SELECT
                albums.name, albums.id, artists.name, artists.id
            FROM
                albums, artists
            WHERE
                albums.name LIKE '%""" + string + """%' and albums.artist_id=artists.id LIMIT """ + str(self.parent.settings.get_value("PERFORMANCE", "db_search_limit"))
        ret = self.query(sql)
        albums = {}
        counter = 0
        for album in ret:
            albums[counter] = {}
            albums[counter]['album_name'] = album[0]
            albums[counter]['album_id'] = album[1]
            albums[counter]['artist_name'] = album[2]
            albums[counter]['artist_id'] = album[3]
            counter += 1
        return albums

    ###################################################################
    #
    # ????
    #
    def gettags(self, id):
        
        sql = """
            SELECT
                tags
            FROM
                albums
            WHERE
                id=%s
            """ % str(id)
        ret = self.query(sql)
        return ret

#    def albuminfos(self, id):
#        strQuery = "track/id/album/data/json/ID?ali=full&ari=full+object&tri=full&item_o=track_no_asc&showhidden=1&shownotmod=1"
#        strQuery = strQuery.replace("ID", str(id))
#        ret = self.__queryold__(strQuery)
#        return ret







#    def albumtracks(self, id): 		
#        ret = self.get("track_id+track_name+track_stream+track_duration+album_name", "track", "album_track/?album_id=" + str(id)+"&streamencoding=ogg2") 		
#        return ret
#        

#    def artistalbums(self, idstr):
#        ret = self.get("album_id+album_name+album_playlist", "artist", "?idstr=" + idstr + "&ali=full")
#        print ret
#        return ret

#    def artistlist(self):
#        ret =  self.get("id+idstr+name", "artist", "?artist_hasalbums&n=50")
#        return ret
