#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

#JSon parser
import simplejson as json

import os, md5, time

# URL Handling
from urllib2 import urlopen, URLError
from urllib import urlencode as _urlencode

#own classes
import functions

# Gettext - Übersetzung
functions.translation_gettext()
#def _(string):
#    return string

###################################################################
#
# several urls for jamendo - should be moved to documentation
#
### ari = Artistinfo
### tri = Trackinfo
### ali = albuminfo
# ===> http://developer.jamendo.com/de/wiki/MusiclistApi_draft
# info_min or info_common or full.

### ALLE MÖGLICHEN ALBEN INFOS:
    #http://www.jamendo.com/get/album/id/album/data/json/33?ali=info_common+archives+tracker+lengths&ari=info_min&aenc=ogg3

### Alle Tracks eines Albums:
    # http://www.jamendo.com/get/track/id/album/data/json/33?ali=full&ari=full+object&tri=full&item_o=track_no_asc&showhidden=1&shownotmod=1
    
### Alle Infos zum Track:
    # http://www.jamendo.com/get/track/id/track/data/json/4707?ali=full&ari=full+object&tri=full&item_o=track_no_asc&showhidden=1&shownotmod=1

### 50 populäre alben von rock
   # http://api.jamendo.com/get2/id+name+url+image+artist_name/album/jsonpretty/?tag_idstr=rock&n=50&order=rating_desc

### Torrent, redirect
    # http://api.jamendo.com/get2/bittorrent/file/redirect/?album_id=919&type=archive&class=ogg3

class Jamendo:
    def __init__(self, parent):
        self.parent = parent
        self.home = functions.preparedirs()
        self.cachedir = os.path.join(self.home, "cache")
        self.cache_counter = 0
        self.jamendo_counter = 0

        self.last_query = None
        
        # Somehow simplejsons behaviour
        # changed some times so i had to
        # do it this way. 
        # If this new behaviour won't change
        # again, the strict-statement is
        # obsolete
        try:
            self.json = json.JSONDecoder(strict=False)
        except:
            self.json = json.JSONDecoder()

    def json_decode(self, string):
        try:
#            return simplejson.loads(string.decode('utf-8'))
#            x= json.read(string)

            return self.json.decode(string) 
        except Exception, inst:
            desc = "An error occured while parsing Jamendos server-response\n"
            desc += "Most probably this is a json issue coming up with new\n"
            desc += "versions of debian.\n"
            desc += "<b>Get the newest version of pyjama to deal with it!</b>"
            self.parent.Events.raise_event("error", inst, desc)
            return None

    ###################################################################
    #
    # do a jamendo query
    # RETURNS: dictionary
    #
    def get(self, field, unit, params, format="json"):
        print ("Requesting database on jamendo.com")
        query = "http://api.jamendo.com/get2/" + field + "/" + unit + "/" + format  + "/" + params
        if self.parent.debug:
            print query
        source = urlopen(query)
        print ("Requesting database - done")
        return self.json_decode(source.read())

    ###################################################################
    #
    # tests if a specific query can be found in cache and if it is up to date
    # RETURNS: False if not in cache or to old - otherwise the content of cached query
    #
    def cache(self, query, CACHING_TIME = None):
        if CACHING_TIME == None or CACHING_TIME == self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG"):
            CACHING_TIME = self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG")
            subdir = "long"
        else:
            subdir = "short"
        md5hash = md5.new(query).hexdigest()
        datei = os.path.join(self.cachedir, subdir, md5hash)
        if os.path.exists(datei):
            file_time = os.stat(datei).st_mtime
            if file_time + CACHING_TIME > time.time():
                datei = open(datei,'r')
                content = datei.read()
                datei.close()
                remaining = CACHING_TIME - (time.time() - file_time)
                m = 60
                h = 60*m
                d = h*24
                #w = d*7
                days = remaining // d
                hours = (remaining - d*days) // h
                mins = ((remaining - d*days) - h*hours) // m
                secs = (((remaining - d*days) - h*hours) - m*mins)
                if self.parent.verbose:
                    print ("This Query was loaded from Cache. New Data will be loaded from Jamendo  in %i days, %i hourse, %i minutes and %i seconds.") % (days, hours, mins, secs)
                    if CACHING_TIME == self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG"):
                        print ("Don't mind this might be to long. This specific data- set won't change!")
#                self.parent.window.sbStatus.set_text("Info", _("Aus dem Cache. Gültig noch %iT, %i:%i:%i") % (days, hours, mins, secs))
                self.parent.window.sbStatus.set_text("Info", _("From Cache. Reload in %iD, %i:%i:%i") % (days, hours, mins, secs))
                self.cache_counter += 1
                return content
            print ("Cache too old, Loading Data from Jamendo")
            self.parent.window.sbStatus.set_text("Info", _("Refreshing Cache"))
            return False
        print ("No Data cached, yet. Caching Query now")
        self.parent.window.sbStatus.set_text("Info", _("Cached from Jamendo"))
        self.jamendo_counter += 1
        return False

    ###################################################################
    #
    # creates cache for a query
    # RETURNS: n/A
    #
    def write2cache(self, query, result, caching_time = None):
        if caching_time == None or caching_time == self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG"):
            subdir = "long"
        else:
            subdir = "short"
        if self.parent.debug:
            print subdir
        md5hash = md5.new(query).hexdigest()
        datei = os.path.join(self.cachedir, subdir, md5hash)
        fh = file(datei,'w')
        fh.write(result)
        fh.close()

    ###################################################################
    #
    # queries Jamendo
    # RETURNS: dictionary
    #
    def query(self, query, caching_time = None):
        query = "http://api.jamendo.com/get2/" + query
        ret = self.cache(query, caching_time)
        if  ret != False: ##wenn cache daten vorliegen
            return self.json_decode(ret)
        if self.last_query == None:
            self.last_query = time.time()
        elif self.last_query + 1 > time.time():
            print "To fast"
            return None
        else:
            self.last_query = time.time()
        if self.parent.verbose:
            print ("Requesting database on jamendo.com")
        if self.parent.debug:
            print query
        try:
            source = urlopen(query)
        except URLError, inst:
            desc = "Could not query Data from jamendo.\n"
            desc += "Perhaps Jamendo is maintening the servers\n"
            desc += "or your connection is broken."
            self.parent.Events.raise_event("error", inst, desc)
            print ("couldn't load from jamendo")
            return -1
        ret = source.read()
        self.write2cache(query, ret, caching_time)
        if self.parent.verbose:
            print ("Requesting database - done, wrote cache")
        return self.json_decode(ret)

    ###################################################################
    #
    # queries Jamendo's old API
    # RETURNS: dictionary
    #
    def queryold(self, query, caching_time = None): 
        query = "http://www.jamendo.com/get/" + query
        ret = self.cache(query, caching_time)
        if  ret != False: ##wenn cache daten vorliegen
            return self.json_decode(ret)
        if self.last_query == None:
            self.last_query = time.time()
        elif self.last_query + 1 > time.time():
            print ("To fast")
            return None
        else:
            self.last_query = time.time()   
        print ("Requesting database on jamendo.com")
        if self.parent.debug:
            print query
        try:
            source = urlopen(query)
        except URLError, inst:
            desc = "Could not query Data from jamendo.\n"
            desc += "Perhaps Jamendo is maintening the servers\n"
            desc += "or your connection is broken."
            self.parent.Events.raise_event("error", inst, desc)
            print ("couldn't load from jamendo")
            return -1
        ret = source.read()
        self.write2cache(query, ret, caching_time)
        print ("Requesting database - done, wrote cache")
        return self.json_decode(ret)

    ###################################################################
    #
    # get NUM top ORDER albums from Jamendo out of TAG
    # RETURNS: dictionary
    #
    def top(self, num = 10, order = "tag", page=1, tag="all"):
        if tag == "all": 
            if self.parent.verbose:
                print ("Getting top %i ordered by '%s'") % (num, order)
            ret = self.query("artist_name+artist_idstr+artist_id+album_name+album_id+album_image+album_genre/album/json/?order=%s_desc&n=%i&pn=%s" % (order, num, page), self.parent.settings.get_value("JAMENDO", "CACHING_TIME_SHORT"))
        else:
            if self.parent.verbose:
                print ("Getting %i albums tagged '%s' ordered by '%s'") % (num, tag, order)
            ret = self.query("artist_name+artist_idstr+artist_id+album_name+album_id+album_image+album_genre/album/json/?tag_idstr=%s&n=%i&pn=%s&order=%s_desc" % (tag, num, page, order), self.parent.settings.get_value("JAMENDO", "CACHING_TIME_SHORT"))
        return ret

    ###################################################################
    #
    # get stream URL for a track
    # RETURNS: string
    #
    def stream(self, id):
        if self.parent.verbose:
            print ("Getting stream informations")
        ret = self.query("stream/track/json/?id=%i&streamencoding=ogg2" % id, self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG"))
        if ret == None:
            return None
        return ret[0]
                                #track/id/album/data/json/15395?ali=common_info+archives+artist&ari=full+object&tri=full&item_o=track_no_asc&showhidden=1&shownotmod=1

    ###################################################################
    #
    # get infos for a track
    # RETURNS: dictionary
    #
    def trackinfos(self, id):
        if self.parent.verbose:
            print ("Getting track informations")
        ret = self.queryold("track/id/track/data/json/%s?ali=full&ari=full+object+stream&tri=full&item_o=track_no_asc&showhidden=1&streamencoding=ogg2&shownotmod=1" % str(id), self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG"))
        return ret

        #!!!! ret = self.queryold("track/id/album/data/json/%s?ali=full&ari=full+object&tri=full&item_o=track_no_asc&showhidden=1&shownotmod=1" % id) 

    ###################################################################
    #
    # old
    # RETURNS: dictionary
    #
    def albuminfos_altundmaechtig(self, id):
        strQuery = "track/id/album/data/json/ID?ali=full&ari=full+object&tri=full&item_o=track_no_asc&showhidden=1&shownotmod=1"
        strQuery = strQuery.replace("ID", str(id))
        ret = self.queryold(strQuery, self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG"))
        return ret

    ###################################################################
    #
    # get infos for a album
    # RETURNS: dictionary
    #
    def albuminfos(self, id):
        if self.parent.verbose:
            print ("Getting album informations")
        strQuery = "album/id/album/data/prettyjson/ID?ali=full&ari=full+object&item_o=track_no_asc&showhidden=1&shownotmod=1"
        strQuery = strQuery.replace("ID", str(id))
        ret = self.queryold(strQuery, self.parent.settings.get_value("JAMENDO", "CACHING_TIME_LONG"))
        if ret == None:
            return None
        return ret[0]




#http://api.jamendo.com/get2/track_name+track_duration+track_url+license_url+album_id+album_name+artist_id+artist_idstr+artist_name/track/jsonpretty/album_artist/

    ###################################################################
    #
    # get an album's tracks
    # RETURNS: dictionary
    #
    def albumtracks(self, id): 		
        ret = self.get("track_id+track_name+track_stream+track_duration+album_name", "track", "album_track/?album_id=" + str(id)+"&streamencoding=ogg2") 		
        return ret
        
    ###################################################################
    #
    # get an artist's album 
    # RETURNS: dictionary
    # 
    def artistalbums(self, idstr):
        ret = self.get("album_id+album_name+album_playlist", "artist", "?idstr=" + idstr + "&ali=full")
        print ret
        return ret

    ###################################################################
    #
    # get a list of artists
    # RETURNS: dictionary
    #
    def artistlist(self):
        ret =  self.get("id+idstr+name", "artist", "?artist_hasalbums&n=50")
        return ret
