#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

######################################################################
#                                                                    #
#                     Some default Layouts                           #
#                                                                    #
######################################################################    
#
# Album Layout - shows an album
#

import gtk
import os
import md5
import urllib
import time

import clWidgets
import functions

class AlbumLayout(gtk.Layout):

    def __init__(self, pyjama):
        self.pyjama = pyjama
        
        gtk.Layout.__init__(self)
        self.set_size(700,400)

        # Labels shown in Album- View
        self.AlbumLabels={}

        # Labels on Album Widget
        y = 10
        xspace = 250
        yspace = 20
        self.AlbumInfos = ['name', 'artist_name', 'trackcount', 'lengths', 'genre', 'releasedate', 'review_note', 'url']
        self.AlbumCaptions = [_("Album's title"), _("Artist"), _("#Tracks"), _("Length"), _("Genre"), _("Releasedate"), _('Rating'), _('link')]
        for info in xrange(0, len(self.AlbumInfos)):
            if self.AlbumInfos[info] == "url":
                self.AlbumLabels[self.AlbumInfos[info]] = gtk.LinkButton("", _("link")) #str(album['album_name'][:20])
                self.put(self.AlbumLabels[self.AlbumInfos[info]], xspace, y)
                self.AlbumLabels[self.AlbumInfos[info]].connect("clicked", self.on_zumAlbum_clicked)
                self.AlbumLabels[self.AlbumInfos[info]].show()
            else:
                self.AlbumLabels[self.AlbumInfos[info]] = gtk.Label("")
                self.AlbumLabels[self.AlbumInfos[info]].set_single_line_mode(True)
                self.put(self.AlbumLabels[self.AlbumInfos[info]], xspace, y)
                self.AlbumLabels[self.AlbumInfos[info]].show()
            if self.AlbumInfos[info] == "review_note":
                # USER WIDGET Rating
                self.rtRating = clWidgets.Rating()
                self.put(self.rtRating, 350, y)
                self.rtRating.show() # for some reason this is not done by self.show_all()
            y+=yspace

        self.image_album = gtk.Image()
        self.put(self.image_album, 30, 10)
        self.image_album.show()

        # might be obsolet
        self.pyjama.window.setcolor(self)
        self.show_all()

    def on_zumAlbum_clicked(self, ev):
        url = self.AlbumLabels['url'].get_label()
        self.pyjama.Events.raise_event("open_url", url)


    def draw(self, data1, data2, data3, data4):
        #
        # Setting label
        #
        markup = self.pyjama.window.markuplbCaption.replace("TEXT", _("Show album infos for '%s'") % data1['name'])
        self.pyjama.window.lbCaption.set_markup(markup)

        self.toolbar = self.pyjama.layouts.toolbars['album']

        albuminfos = data1

        tracks = self.pyjama.db.albumtracks(albuminfos['id'])
        track_count = len(tracks)

        if track_count == 0:
            print ("Album not in database, yet.")
            print albuminfos['public_date']
            dia = clWidgets.MyDialog(_('Album non existant.'),
                              self.window.get_toplevel(),
                              gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
                                (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT), gtk.STOCK_DIALOG_WARNING, _('This album is not in the database.\nPerhaps Jamendo did not unloack that album, yet\n or you are using a old local database.'))
            dia.run()
            dia.destroy()
            self.pyjama.layouts.toolbars['top'].on_bHistoryBack_clicked(None)
            return None


        #tags = albuminfos['id3genre']

        genre = albuminfos['id3genre']
        #desc = albuminfos['desc_str']
        #referrer_count = albuminfos['referrer_count']
            #playlisted_count = albuminfos['playlisted_count']
            #favourited_count = albuminfos['favourited_count']
        #lowfi_count = albuminfos['lowfi_count']
        releasedate = albuminfos['public_date']
        lengths = albuminfos['lengths']
            #license_id = albuminfos['license_id']
            #review_num = albuminfos['review_num']
        review_note = albuminfos['review_note']
            #review_num_week = albuminfos['review_num_week']
            #review_note_week = albuminfos['review_note_week']
        name = albuminfos['name']
        artist_id = albuminfos['artist_id']
        artist_name = tracks[0]['artist_name']
        album_id = albuminfos['id']
            #img = albuminfos['images'][0]['url']
        img = albuminfos['root_images'] + "1.200.jpg"
        url = self.pyjama.settings.get_value("URLs", "ALBUM_URL").replace("URL", "%s" % album_id)
        albuminfos['tracks'] = tracks

        self.pyjama.window.tvList.clear()
        for track in tracks:
            results = [artist_name, name, tracks[track]['numalbum'],tracks[track]['name'], tracks[track]['license'], artist_id, album_id, tracks[track]['id']]
            self.pyjama.window.tvList.add_item(results)


        self.toolbar.lbAppendAlbum.tag = tracks #self.tracks = tracks


        md5hash = md5.new(img).hexdigest()
        fh = os.path.join(self.pyjama.home, "images", md5hash)
        if not os.path.exists(fh):
            try:
                urllib.urlretrieve(img, fh)
            except IOError:
                print ("Could not load image")
                return None


        self.image_album.clear()
        if img == "" or img == None:
            pixbuf = self.image_album.render_icon(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_DIALOG, detail=None)
            self.image_album.set_from_pixbuf(pixbuf)
        else:
            self.image_album.set_from_file(fh)


        content = [ name, functions.decode_htmlentities(artist_name), track_count, functions.sec2time(lengths), functions.id2genre(genre), releasedate, review_note, url ]
        x = 0
        infos = self.AlbumInfos
        namen = self.AlbumCaptions
        for x in xrange(0, len(infos)):
            if infos[x] == "review_note":
                if self.pyjama.nocolor:
                    self.AlbumLabels[infos[x]].set_markup("<span><b>%s: </b></span>" % (namen[x]))
                else:
                    self.AlbumLabels[infos[x]].set_markup("<span foreground=\"white\"><b>%s: </b></span>" % (namen[x]))
                self.rtRating.set_rating(content[x])
            elif infos[x] == "url":
                    self.AlbumLabels[infos[x]].set_label(url)
            else:
                if self.pyjama.nocolor:
                    self.AlbumLabels[infos[x]].set_markup("<span><b>%s: %s</b></span>" % (namen[x], content[x]))
                else:
                    self.AlbumLabels[infos[x]].set_markup("<span foreground=\"white\"><b>%s: %s</b></span>" % (namen[x], content[x]))


        # NEW ATTRIBUTE 'FLAG' IN STOCKBUTTON
        # NEED TO USE THIS FOR OTHER BUTTONS
        # HERE, TOO!
        self.toolbar.sbGotoWeb.tag = url
        
        self.toolbar.lbMoreAlbumsFromThisArtist2.tag = artist_id
        self.toolbar.lbMoreAlbumsFromThisArtist2.set_tooltip_text(_("Show informations and albums from '%s'") % artist_name)
        self.toolbar.sbDownloadAlbum.tag = album_id

        self.set_size(self.pyjama.window.scrolledwindow_width-20,230)#350)#y+200
        self.pyjama.Events.raise_event("showing_album_page", albuminfos)

    #
    # Actually this toolbar is just a hbox...
    #
    class ToolBar(gtk.HBox):
        def __init__(self, pyjama):
            gtk.HBox.__init__(self)
            self.pyjama = pyjama
            self.layout = self.pyjama.layouts.layouts['album']

            # ADD ALBUM TO PLAYLIST
            self.lbAppendAlbum = clWidgets.StockButton(gtk.STOCK_ADD, gtk.ICON_SIZE_DND)
            self.lbAppendAlbum.set_tooltip_text(_("Append this album on playlist"))
            self.lbAppendAlbum.show()
            self.pack_start(self.lbAppendAlbum, False, True, 2)
            self.lbAppendAlbum.connect("clicked", self.on_lbAppendAlbum_clicked)
            # GET MORE ALBUMS FROM THIS PLAYLIST
            self.lbMoreAlbumsFromThisArtist2 = clWidgets.StockButton(gtk.STOCK_INFO, gtk.ICON_SIZE_DND) 
            self.pack_start(self.lbMoreAlbumsFromThisArtist2, False, True, 2)
            self.lbMoreAlbumsFromThisArtist2.connect("clicked", self.on_lbMoreAlbumsFromThisArtist_clicked)
            self.lbMoreAlbumsFromThisArtist2.show()
            # GO TO ALBUM'S PAGE ON JAMENDO
            self.sbGotoWeb = clWidgets.StockButton(gtk.STOCK_NETWORK, gtk.ICON_SIZE_LARGE_TOOLBAR)
            self.sbGotoWeb.set_tooltip_text(_("Goto to album's page on jamendo"))
            self.pack_end(self.sbGotoWeb, False, True, 2)
            self.sbGotoWeb.set_size_request(50,50)
            self.sbGotoWeb.show()
            self.sbGotoWeb.connect("clicked", self.on_sbGotoWeb_clicked)        
            # DOWNLOAD ALBUM VIA TORRENT
            self.sbDownloadAlbum = clWidgets.StockButton(gtk.STOCK_GOTO_BOTTOM, gtk.ICON_SIZE_LARGE_TOOLBAR)
            self.sbDownloadAlbum.set_tooltip_text(_("Download Album as Torrent"))
            self.sbDownloadAlbum.set_size_request(50,50)
            self.sbDownloadAlbum.show()
            self.pack_end(self.sbDownloadAlbum, False, True, 2)
            self.sbDownloadAlbum.connect("clicked", self.on_sbDownloadAlbum_clicked)    

        def on_lbMoreAlbumsFromThisArtist_clicked(self, ev):
            # query db for artist informations and more albums
            self.pyjama.start_pulsing(text = _("Requesting local database"))
            ret = self.pyjama.db.artistinfos(self.lbMoreAlbumsFromThisArtist2.tag, time.time())        
            self.pyjama.stop_pulsing()
            self.pyjama.layouts.show_layout("artist", ret, who_called = "on_lbMoreAlbumsFromThisArtist_clicked")


        def on_lbAppendAlbum_clicked(self, ev):
            tracks = self.lbAppendAlbum.tag
            for track in tracks: # for track in self.main.tracks:
                tracks[track]['uid'] = "%f%s" % (time.time(), tracks[track]['id'])
                self.pyjama.add2playlist(tracks[track])
                status = self.pyjama.player.status
                if status == "Error" or status == "End" or status == None:
                    self.pyjama.window.on_bPlay_clicked(None)

        def on_sbGotoWeb_clicked(self, ev):
            self.pyjama.Events.raise_event("open_url",  self.sbGotoWeb.tag)

        def on_sbDownloadAlbum_clicked(self, ev):
            url = "http://api.jamendo.com/get2/bittorrent/file/redirect/?album_id=%s&type=archive&class=%s" % (self.sbDownloadAlbum.tag, self.pyjama.settings.get_value("JAMENDO", "FORMAT"))
            self.pyjama.Events.raise_event("open_url", url, force_default=True)        


