#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# This is pyjama's main class. It holds allmost every other class, which
# is needed. If you are writing plugins, this main class will be passed
# to you plugin's __init__ function!
# ----------------------------------------------------------------------------

# math
from math import floor
# Time formatting
from time import strftime, gmtime, time, sleep

# Laden von Dateien
import urllib
import md5

# Threading
import thread

import traceback 
import os

try:
    import gnome.ui
    GNOME_UI_AVAILABLE = True
except:
    GNOME_UI_AVAILABLE = False

#RegEx
import re

import functions
import clSettings
import clJamendo, clGstreamer010, clDB, clThreadedDownload #clPlayer
from clWidgets import *
import clEntry#, clProgressWindow
import clPlugin, clEvent
import notification
import clBrowserInterface
import download_db
import clLayouts

class main():
    def __init__(self, parent, options):
        self.version = functions.VERSION
        if GNOME_UI_AVAILABLE:
            gnome.init("Pyjama", self.version)
        print "PYJAMA FOUND IN %s" % functions.install_dir()


        # USER CLASS OPTIONS
        self.settings = clSettings.settings(self)
                    

        # USER CLASS EVENTS
        self.Events = clEvent.Events()
        # Create some events:
        self.Events.add_event("pluginloaded")
        self.Events.add_event("nowplaying")
        self.Events.add_event("alldone")
        self.Events.add_event("showing_album_page")
        self.Events.add_event("showing_artist_page")
        self.Events.add_event("firstrun")
        self.Events.add_event("error")
        self.Events.add_event("layout_changed")
        self.Events.add_event("scrolled_window_resized")

        self.Events.add_event("albuminfo_created")


#        self.Events.pluginloaded += self.ev_plugin_loaded
#        self.Events.eventlist['pluginloaded'] +=
        self.Events.connect_event("pluginloaded", self.ev_plugin_loaded)

        # Function called when an error occures:
        #self.error_fkt = self.error
        self.Events.connect_event("error", self.error)

        # Database- Tools
        self.dump_tools = download_db.dump_tools(self)
        # GTK WINDOW
        self.window = parent
        # USER CLASS TRAY ICON
        self.icon = notification.TrayIcon(self)
        # USER CLASS PLAYER
        self.player = clGstreamer010.Player(self)
        # USER CLASS JAMENDO FOR JAMENDO QUERIES
        self.jamendo = clJamendo.Jamendo(self)
        # USER CLASS LOCAL DATABASE
        self.db = clDB.DB(self)
        # USER CLASS BROWSER
        self.browser = clBrowserInterface.Browser(self)
        # USER CLASS LAYOUTS
        self.layouts = clLayouts.Layouts(self)
    



        # VERBOSE AND / OR DEBUGGING
        self.verbose = options.verbose
        self.debug = options.debug
        self.debug_extreme = options.debug_extreme
#        self.print_tracebacks = options.print_tracebacks
        
        # REDUCES COLORS
        self.nocolor = options.theme != None or options.nocolor
        # SWITCHES EVERY 200 MS, really ugl, but got
        # no better ideo doing that
        self.allow_rearrange = True

        # Homedir
        self.home = functions.preparedirs()
        # Iters for Playlist Treeview
        self.tvMarkedItem = None

        
        # Tracks currently shown in draw_albumdetail
        # This was replaced through StockBotton- Tags!
        #self.tracks = {}
        
        # Is the Timer running?
#        self.timer = False
        
        
        # Is the Progressbar shown (for pulsing-timer)
        self.pulsing = False
        
        # HISTORY
        self.historyBack = []
        self.historyForward = []
        self.historyCurrent = {}

        # USER CLASS PLUGINS (loaded last, so that all
        #vars and methods are available to the plugins
        self.plugins = clPlugin.Plugins(self)
        if self.settings.get_value("PYJAMA", "FIRST_RUN", True) == True:
            self.Events.raise_event("firstrun")
            self.settings.set_value("PYJAMA", "FIRST_RUN", "False")

        # decided to let the timer run all the time
        self.start_timer()
        
#        self.plugins.loaded['test2'].test()
        

    def reload_current_page(self):
        # save scroll position for scrolledwindow
        adjustment = self.window.scrolledwindow.get_vadjustment()
        self.window.scrolledwindow.hide()
        # reloading page
        data = self.historyCurrent
        self.layouts.show_layout(data['layout'], data['data1'], data['data2'], data['data3'], data['data4'], fromhistory=True, who_called='reload_current_page')
        # load scroll position
        self.window.scrolledwindow.set_vadjustment(adjustment)
        self.window.scrolledwindow.show()

    def error(self, error_inst, desc=False):
            print "An error occured: %s" % error_inst
            tb =  traceback.format_exc()
            print "-"*60
            print tb
            print "-"*60
            if not desc: 
                desc = error_inst

    def ev_plugin_loaded(self, name, version, order, mod_name):# plugin_name, module_name):
#        if self.verbose:
        print ("Plugin '%s' V%s (%s) loaded") % (name, version, mod_name)

    def set_allow_rearrange(self):
        self.allow_rearrange=True
        return False

    def check_next_possible(self, page=None, rpp=None):
        if page == None: page = self.cur_page
        if rpp == None: rpp = self.results_per_page
        if page >= floor(self.db.albums / rpp) - 2:
            self.window.sbNextPage.set_sensitive(False)
        else:
            self.window.sbNextPage.set_sensitive(True)

    def check_prev_possible(self, page=None):
        if page == None: page = self.cur_page
        if page == 1:
            self.window.sbPrevPage.set_sensitive(False)
        else:
            self.window.sbPrevPage.set_sensitive(True)
            

    def appendtracks(self, tracks, play = False):
        cur = len(self.player.playlist)
        for track in tracks:
            tracks[track]['uid'] = "%f%s" % (time(), tracks[track]['id'])
            self.add2playlist(tracks[track])
            status = self.player.status
        if play:
            self.window.on_bStop_clicked(None)
            self.setplaylist(cur)
            self.window.on_bPlay_clicked(None)        

    def remove_item_from_playlist(self, item):
        if item < self.tvMarkedItem:
            self.tvMarkedItem -= 1
        elif item == self.tvMarkedItem:
            self.tvMarkedItem = None
        self.player.remove(item)

    def move_playlist_item(self, s, d, before): # sourcepath, destpath
        c = self.tvMarkedItem
        if d > s:
            if before: 
                bf = -1
            else:
                bf = 0
        else:
            if before: 
                bf = 0
            else:
                bf = 1
        dest = d + bf


        self.tvMarkedItem = dest
        self.player.move_item(s, d, before)

    def add2playlist(self, track):
        self.player.add2playlist(track)
        
        artist_name = track['artist_name']
        track_name = track['name']
        markup = self.window.markupNormalEntry.replace("__ARTIST__", artist_name).replace("__TITLE__", track_name).replace("__NUM__", str(track['numalbum']))
        tmpIter = self.window.liststore.append([markup]) #, track['id']


    def showInfo(self, path=None):
        if path == None:
            if self.tvMarkedItem != None and self.tvMarkedItem < len(self.player.playlist):
                path = self.tvMarkedItem
            else:
                return None
#        if path == None:
#            ret = self.window.tvPlaylistSelection.get_selected()
#            if ret != None:
#                model, retIter = ret
#                if retIter != None:
#                    path = model.get_path(retIter)[0]
#                else:
#                    return None
#            else:
#                return None
            
        track = self.player.playlist[path]
        #Covers = simplejson.loads(track['album_Covers'])

        img = self.get_album_image(track['album_id'])
        if not img: return None
        self.window.imgCover.set_from_file(img)
        self.window.lbCoverArtist.set_text(track['artist_name'])
        self.window.lbCoverAlbum.set_text(track['album_name'])

    def setplaylist(self, activate_item):
        self.window.pbSong.set_fraction(0)
        self.window.pbSong.set_text("")

        model = self.window.tvPlaylist.get_model()
        retIter = model.get_iter(activate_item)
        self.showInfo(activate_item)

        # Unmark last played Item
        if self.tvMarkedItem != None and self.player.last_played != None and self.tvMarkedItem < len(self.player.playlist):
            track = self.player.last_played
            artist_name = track['artist_name']
            track_name = track['name']
            track_numalbum = str(track['numalbum'])
            markup = self.window.markupNormalEntry.replace("__ARTIST__", artist_name).replace("__TITLE__", track_name).replace("__NUM__", track_numalbum)
            self.window.liststore.set(model.get_iter(self.tvMarkedItem), 0, markup)

        # Mark cur. played item
        
        self.window.tvSelection.select_iter(retIter)
        track = self.player.playlist[activate_item] #self.player.cur_playing
        artist_name = track['artist_name']
        track_name = track['name']
        track_numalbum = str(track['numalbum'])
        markup = self.window.markupCurPlaying.replace("__ARTIST__", artist_name).replace("__TITLE__", track_name).replace("__NUM__", track_numalbum)
        self.window.liststore.set(retIter, 0, markup)
        # remember, which item is marked
        self.tvMarkedItem = activate_item

    def stop_timer(self):
        pass
        
    def start_timer(self):
        gobject.timeout_add(200, self.timer_event)
#        self.timer = True

    def start_pulsing(self, text = ""):
        gobject.timeout_add(50, self.pulse)
        self.window.pbWait.set_fraction(0)        
        self.window.pbWait.set_text(text)
        self.window.pbWait.show()
        
    
    def pulse(self):
        self.window.pbWait.pulse()
        if self.pulsing:
            return True
        else:
            return False
    
    def stop_pulsing(self):
        self.pulsing = False
        self.window.pbWait.hide()
    
    def timer_event(self):
        #self.window.pbWait.pulse()
        self.player.check_status()
        status = self.player.status
        status = _(status)
        fehler = ["Error", "Stop", "End", None]
        self.setStatus( status )
        if status == "paused":
            self.window.bPlay.setimage(gtk.STOCK_MEDIA_PLAY)
            self.window.bPlay.tag = "play"
            if self.window.pbSong.counter > 2:
                if self.window.pbSong.get_text() == _("paused"):
                    if self.window.pbSong.ShowRemainingTime:
                        txt =  "-%s" % functions.sec2time(self.player.togo*-1)
                    else:
                        txt = "%s/%s" % (functions.sec2time(self.player.cursec), functions.sec2time(self.player.duration))
                else:
                    txt = _("paused")
                self.window.pbSong.set_text(txt)
                self.window.pbSong.counter = 0
            self.window.pbSong.counter +=1
            return True
        elif not status in fehler and self.player.percentage != None:
            if self.window.bPlay.tag != "pause":
                self.window.bPlay.setimage(gtk.STOCK_MEDIA_PAUSE)
                self.window.bPlay.tag = "pause"
            #print self.player.text
            if self.player.duration > 0 and self.player.duration < 59*60:
                self.window.pbSong.set_fraction(self.player.percentage)            
                if self.window.pbSong.ShowRemainingTime:
                    txt =  "-%s" % functions.sec2time(self.player.togo*-1)
                else:
                    txt = "%s/%s" % (functions.sec2time(self.player.cursec), functions.sec2time(self.player.duration))
            else:
                self.window.pbSong.pulse()
                txt = "%s" % functions.sec2time(self.player.cursec)
            self.window.pbSong.set_text(txt)
            return True
        elif status == "Stop" or status == "End": # and not self.pulsing:
            #self.window.bPlay.setimage(gtk.STOCK_MEDIA_PLAY)
            #self.window.bPlay.tag = "play"
            if self.icon.menu.play_button_status != "play":
                self.icon.menu.switch_play_button("play")
            self.window.pbSong.set_fraction(0)
            self.window.pbSong.set_text("")
#            self.timer = False
            return True
        else:
            return True   

    def setStatus(self, text):
        self.window.sbStatus.set_text("Player", text)

            
    def notification(self, caption, text, icon="pyjama", size = 100):
        self.icon.show_notification(caption, text, icon, size)

    def switch_window_state(self, ev1):
        if self.window.window.get_state() & gtk.gdk.WINDOW_STATE_ICONIFIED:
            self.window.deiconify()
        else:
            self.window.iconify()

    def get_album_image(self, album_id, size = 100):
        img = "http://imgjam.com/albums/%s/covers/1.%i.jpg"  % (album_id, size)
        md5hash = md5.new(img).hexdigest()
        fh = os.path.join(self.home, "images", md5hash)
        if not os.path.exists(fh):
            try:
                urllib.urlretrieve(img, fh)
            except IOError:
                print ("Could not load image")
                return None
        return fh
