#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2009 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import gtk

######################################################################
#                                                                    #
#                               Menuitem                             #
#                                                                    #
######################################################################
# my menubar
# Has some extra functions for placing root menus at specific positions
class Menu(gtk.MenuBar):
    def __init__(self, window):
        self.pyjama = window.main

        gtk.MenuBar.__init__(self)

        fake_array_for_translations = [
            _("File"), _("Extras"), _("Help"),       # Root menus
            _("Save"), _("Quit"),                   # Sub File
            _("About"), _("Help")                   # Sub Edit
        ]

        menus = ["File", "Extras", "Help"]  
        self.rootmenus = []
        self.submenus = []
        
        for menu in menus:
            self.append_root(_(menu), menu)

        sub1 = ["Quit"]  
        self.create_submenu(rootmenu=self.get_rootmenu("File"), submenu=sub1)

        sub2 = []
        self.create_submenu(rootmenu=self.get_rootmenu("Extras"), submenu=sub2)

        sub3 = ["Help", "---", "About"]  
        self.create_submenu(rootmenu=self.get_rootmenu("Help"), submenu=sub3)

        # 
        # Connect
        #
        about = self.get_submenu("About")
        about.connect("activate", self.pyjama.window.show_about)
        help = self.get_submenu("Help")
        help.connect("activate", self.ev_help)
        quit = self.get_submenu("Quit")
        quit.connect("activate", self.pyjama.window.really_quit)

        #
        # Accelerators
        #
        self.accel_group = gtk.AccelGroup()
        self.get_submenu("Quit").add_accelerator("activate", self.accel_group, ord("q"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
        self.get_submenu("Help").add_accelerator("activate", self.accel_group, gtk.keysyms.F1, 0, gtk.ACCEL_VISIBLE)
        self.pyjama.window.add_accel_group(self.accel_group)
        

        self.show()

    #
    # Finding a menu
    #
    def get_rootmenu(self, name):
        for menu in self.rootmenus:
            if menu.id == name:
                return menu
        return None

    def get_submenu(self, name):
        for menu in self.submenus:
            if menu.id == name:
                return menu
        return None

    #
    # Root Menu Placing
    #
    def append_root(self, name, id):
        mnu = gtk.MenuItem(name)
        mnu.id = id
        mnu.position = len(self.rootmenus)
        self.append(mnu)
        mnu.show()
        self.rootmenus.append(mnu)
        return mnu

    def prepend_root(self, name, id):
        mnu = gtk.MenuItem(name)
        mnu.id = id
        mnu.position = len(self.rootmenus)
        self.prepend(mnu)
        mnu.show()
        self.rootmenus.insert(0, mnu)
        return mnu

    def insert_root_pos(self, pos, name, id):
        mnu = gtk.MenuItem(name)
        mnu.id = id
        mnu.position = pos
        self.insert(mnu, pos)
        mnu.show()
        self.rootmenus.insert(pos, mnu)
        return mnu

    def insert_root_after(self, after, name, id):
        after = self.get_rootmenu(after)
        if after != None:
            return self.insert_root_pos(after.position + 1, name, id)
        return None

    def insert_root_before(self, before, name, id):
        before = self.get_rootmenu(before)
        if before != None:
            if before.position != 0:
                return self.insert_root_pos(before.position , name, id)
            else:
                return self.prepend_root(name, id)
        return None
    
    #
    # Submenu
    #
    def create_submenu(self, rootmenu, submenu):
        mnu = gtk.Menu()
        for sub in submenu:
            if sub != "---":
                tmp = gtk.MenuItem(_(sub))
                tmp.id = sub
            else:
                tmp = gtk.SeparatorMenuItem()
                tmp.id = None
            mnu.append(tmp)
            tmp.show()
            self.submenus.append(tmp)
        mnu.show()
        rootmenu.set_submenu(mnu)
        return mnu

    def append_entry(self, rootmenu, entry, id):
        if entry != "---":
            mnu = gtk.MenuItem(entry)
            mnu.id = id
        else:
            mnu = gtk.SeparatorMenuItem()
            mnu.id = id
        sub = rootmenu.get_submenu()
        sub.append(mnu)
        mnu.show()
        return mnu

    #
    # Help  
    # 
    def ev_help(self, widget):
        self.pyjama.Events.raise_event("open_url", "https://answers.launchpad.net/pyjama", force_default=True)
