#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2009 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

# This Script was inspired by:
# http://pytute.blogspot.com/2007/04/python-plugin-system.html
# Over the time I made several changes which bloat the code a bit
# but gives the opportunity to influence the order of the plugins

import os
import sys
import imp
#import clEvent
import traceback

import functions



class Plugins():
    def __init__(self, pyjama):
        self.pyjama = pyjama
        self.loaded = {}
        self.pluginpath = os.path.join(os.path.dirname(imp.find_module("pyjama")[1]), "plugins/")
        print "\nFound Plugins in folder: ", self.pluginpath
        #self.plugins = [fname for fname in sorted(os.listdir(self.pluginpath)) if os.path.isdir(os.path.join (self.pluginpath, fname)) and os.path.exists(os.path.join(self.pluginpath, fname, "%s.info" % fname))]


        #
        # Listing plugins not supported anymore
        #
        try:
            files = os.listdir(self.pluginpath)
            if ".py" in "".join(files):
                print "############################################################"
                print "#                                                          #"
                print "#  Pyjama uses a new plugin system. For this reason, all   #"
                print "#  files in your plugin folder can be deleted. Please      #"
                print "#  delete the files listed below to save your disk space.  #"
                print "#                                                          #"
                print "# !!The folders in the plugin dir should NOT be deleted!!  #"
                print "#                                                          #"
                print "############################################################"
            for dat in files:
                dat_pos = os.path.join(self.pluginpath, dat)
                if not os.path.isdir(dat_pos) and dat != "readme.txt":
                    print dat_pos
            print "\n\n\n"
        except:
            pass

        # Sine you cannot import from filenames
        # with python, I put the plugin path
        # at the first place in the pathlist
        # after all modules are imported, this
        # entry is removed again. (last line in this module)
        sys.path.insert(0, self.pluginpath[:-1])

        self.plugininfos = []

#        for plugin in self.plugins:
        for plugin in sorted(os.listdir(self.pluginpath)):
            if os.path.isdir(os.path.join(self.pluginpath, plugin)) and os.path.exists(os.path.join(self.pluginpath, plugin, "%s.info" % plugin)) and not plugin+"-" in sys.argv and not plugin in self.pyjama.settings.get_value("PLUGINS", 'blacklist') or plugin+"+" in sys.argv:
                fh = open(os.path.join(self.pluginpath, plugin, "%s.info" % plugin))
                content = fh.read()
                fh.close()
                infos = content.split("\n")

                item_infos = {}
                item_infos['filename'] = plugin

                for line in infos:
                    cur_item = line[:line.find("=")].strip().lower()
                    if cur_item == "name":
                        item_infos['name'] = line[line.find("=")+1:].strip()
                    elif cur_item == "version": 
                        item_infos['version'] = line[line.find("=")+1:].strip()
                    elif cur_item == "order":
                        item_infos['order'] = line[line.find("=")+1:].strip()
                self.plugininfos.append(item_infos)

        #
        # Sorting Plugins by Order
        #
        for x in range(len(self.plugininfos)):
            for plugin in range(0, len(self.plugininfos)-1):
                if int(self.plugininfos[plugin]['order']) > int(self.plugininfos[plugin+1]['order']):
                    self.plugininfos[plugin], self.plugininfos[plugin+1] = self.plugininfos[plugin+1], self.plugininfos[plugin]

        #
        # Importing all plugins
        #
        self.imported_modules = []
        for plugin in self.plugininfos:
            name = plugin['name']
            filename = plugin['filename']
            order = plugin['order']
            version = plugin['version']
            if not name+"-" in sys.argv and not name in self.pyjama.settings.get_value("PLUGINS", 'blacklist') or name+"+" in sys.argv:
                try:
                    mod = __import__ (plugin['filename'])
#                    if not self.pluginpath in mod.__file__:
#                        print "Error"
                    self.imported_modules.append({'module':mod, 'name':name, 'order':order, 'version':version})
                except Exception:
                    print traceback.format_exc()
                    print "Error importing Plugin %s (%s)" % (name, filename)
            else:
                print "Plugin %s ignored" % name

        #
        # Calling main() for each plugin
        #
        ev = self.pyjama.Events
        for plugin in self.imported_modules:
            module = plugin['module']
            name = plugin['name']
            order = plugin['order']
            version = plugin['version']
            mod_name = module.__name__
            try:
                self.loaded[mod_name] = module.main(self.pyjama)
                ev.raise_event('pluginloaded', name=name, version=version, order=order, mod_name=mod_name)
            except:
                print traceback.format_exc()
                print "Error calling Plugin %s (%s)" % (name, mod_name)
                print mod.__file__

        # removing plugin dir form the pathlist
        sys.path.pop(0)


