#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import functions
import ConfigParser, os

class settings():
    def __init__(self, parent=None):
        self.parent = parent
        home = functions.preparedirs()
        install_dir = functions.install_dir()

        self.home_config = os.path.join(home, 'pyjama.cfg')

        self.config = ConfigParser.ConfigParser()
        self.config.readfp(open(os.path.join(install_dir, 'pyjama.cfg')))
        self.config.read([self.home_config])

        self.options = {}

        sections = self.config.sections()
        for section in sections:
            items = self.config.items(section)
            for item, value in items:
                pos = value.find("#")
                pos = value.find(";")
                if pos > -1:
                    value = value[0:pos]
                if value.isdigit():
                    self.options[item] = int(value)
                elif (self.isbool(value)):
                    self.options[item] = self.parsebool(value)
                else:
                    self.options[item] = value

    def section_exists(self, section):
        return self.config.has_section(section)

    def option_exists(self, section, option):
        return self.config.has_option(section, option)

    def set_value(self, section, option, value):
        if not self.section_exists(section):
            self.config.add_section(section)
        self.config.set(section, option, value)
        if not functions.is_writeable(self.home_config):
            try:
                self.parent.Events.raise_event("error", "No write access for %s" % self.home_config)
            except:
                print "No write access for %s" % self.home_config
            return
        fh = open(self.home_config,"w")
        if fh:
            self.config.write(fh)    
        else:
            if self.parent:
                self.parent.Events.raise_event("error", "Error writing configuration to %s" % self.home_config)
            else:
                print "Error writing configuration to %s" % self.home_config
        fh.close()

    def get_value(self, section, option, default=None):
        if not self.section_exists(section): return default
        if not self.option_exists(section, option): return default
        value = self.config.get(section, option)
        pos = value.find("#")
        pos = value.find(";")
        if pos > -1:
            value = value[0:pos]
        if value.isdigit():
            return int(value)
        elif (self.isbool(value)):
            return self.parsebool(value)
        else:
            return value

    ###################################################################
    #
    # returns if a passed string contains a boolean 
    # RETURNS: bool
    # 
    def isbool(self, string):
        if string.upper() == "TRUE" or string.upper() == "FALSE":
            return True
        return False

    ###################################################################
    #
    # returns the boolean value a string contains 
    # RETURNS: bool
    #
    def parsebool(self, string):
        return string[0].upper()=="T"




#x = config.get("JAMENDO", "CACHING_TIME_SHORT") 

#getint 
#getfloat
#getboolean
