#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import gettext
import os, sys
import shutil
from time import strftime, gmtime
import re
import gtk

from constants import *

def install_dir():
    dirname = os.path.dirname(sys.argv[0])
    abspath =  os.path.abspath(dirname)

    if os.name == "nt":
        if os.path.exists(os.path.join(abspath, "clGstreamer010.py")):
            #libs here
            return abspath
        else:
            return os.path.join(os.getenv('PROGRAMFILES'), "pyjama")
    else:
        if os.path.exists(os.path.join(abspath, "clGstreamer010.py")):
            #libs here
            return abspath
        else:
            return INSTALL_DIR_POSIX


###################################################################
#
# Getting the installed pyjama version
# RETURNS: string
#
def get_version():
    try:
        glade=open(os.path.join(install_dir(), "about.glade"), "r").read()
        re1='.*?'	# Non-greedy match on filler
        re2='(property)'	# Word 1
        re3='.*?'	# Non-greedy match on filler
        re4='(name)'	# Word 2
        re5='.*?'	# Non-greedy match on filler
        re6='("version")'	# Double Quote String 1
        re7='.*?'	# Non-greedy match on filler
        re8='((?:(?:[0-2]?\\d{1})|(?:[3][0,1]{1}))[-:\\/.](?:[0]?[0-9]|[1][012])[-:\\/.](?:(?:\\d{1}\\d{1})))(?![\\d])'	# DDMMYY 1

        rg = re.compile(re1+re2+re3+re4+re5+re6+re7+re8,re.IGNORECASE|re.DOTALL)
        m = rg.search(glade)
        if m:
            ddmmyy1=m.group(4)
            return ddmmyy1
        else:
            return "0.0.0"
    except:
        return "0.0.0"

VERSION=get_version()

###################################################################
#
# Getting and making pyjama- dirs
# RETURNS: string (home-directory)
#
def preparedirs(set_privilegs = False):
    home = os.getenv("HOME")
    if home == None: # WINDOWS
        home = os.path.join(os.getenv("HOMEDRIVE"), os.getenv("HOMEPATH"))
    home_pyjama = os.path.join(home, ".pyjama")
    if set_privilegs:
        if os.path.exists(os.path.join(home, ".pyjama")):
            os.system("chmod -R 0777 %s" % os.path.join(home, ".pyjama"))
        if os.path.exists(os.path.join(home, ".pyjama", "images")) == False:
            os.makedirs(os.path.join(home, ".pyjama", "images"))
        if os.path.exists(os.path.join(home, ".pyjama", "cache")) == False:
            os.makedirs(os.path.join(home, ".pyjama", "cache"))
        if os.path.exists(os.path.join(home, ".pyjama", "cache", "long")) == False:
            os.makedirs(os.path.join(home, ".pyjama", "cache", "long"))
        if os.path.exists(os.path.join(home, ".pyjama", "cache", "short")) == False:
            os.makedirs(os.path.join(home, ".pyjama", "cache", "short"))
        if os.path.exists(os.path.join(home, ".pyjama", "themes")) == False:
            os.makedirs(os.path.join(home, ".pyjama", "themes"))
        if not os.path.isfile(os.path.join(home_pyjama, "pyjama.cfg")):
            shutil.copy(os.path.join(install_dir(), "pyjama.cfg"), os.path.join(home_pyjama, "pyjama.cfg"))
            if not os.path.isfile(os.path.join(home_pyjama, "pyjama.cfg")):
                print "Error copying config file to user's pyjama directory"
                print "Please try to do so on your own"
        os.chmod(home_pyjama, 0777)
    return home_pyjama

def get_pyjama_dir():
    return os.path.join(os.getenv("HOME"), ".pyjama")

theme_dirs = [os.path.join(preparedirs(),"themes"), gtk.rc_get_theme_dir(), os.path.join(install_dir(), "themes") ]

###################################################################
#
# Returns a list of available Themes
# RETURNS: tuple
#
def sorteddirlist():
    theme_list = []
    for t in theme_dirs:
        if os.path.exists(t):
            for theme in os.listdir(os.path.join(t)):
                theme_path = os.path.join(t, theme, "gtk-2.0", "gtkrc")
                if os.path.exists(theme_path) and not theme in theme_list:
                    theme_list.append(theme)
    return sorted(theme_list)
    

###################################################################
#
# Theme
# RETURNS: tk.rc_parse(), theme name, 
#
def showtheme(theme):
    home = preparedirs()
    if theme.isdigit():
        dirs = sorteddirlist()
        if int(theme)-1 < len(dirs):
           theme = dirs[int(theme)-1]
        else:
           return _("No theme for entry '%s'") % theme, None
    fl = None
    for t in theme_dirs:
        f = os.path.join(t, theme, "gtk-2.0", "gtkrc")
        if  os.path.exists(f):
            fl = f
    if not fl:
        return _("No theme for entry '%s'") % theme, None
    return gtk.rc_parse(fl), theme


###################################################################
#
# Checks file write permissions for the current user
# RETURNS: bool
#
def is_writeable(path):
    return os.access(path, os.W_OK)

###################################################################
#
# Decoding Display Names
# RETURNS: string
#
from htmlentitydefs import name2codepoint as n2cp
import re

def substitute_entity(match):
    ent = match.group(2)
    if match.group(1) == "#":
        return unichr(int(ent))
    else:
        cp = n2cp.get(ent)

        if cp:
            return unichr(cp)
        else:
            return match.group()

def decode_htmlentities(string):
    entity_re = re.compile("&(#?)(\d{1,5}|\w{1,8});")
    return entity_re.subn(substitute_entity, string)[0]

###################################################################
#
# FORMATING TIME
# RETURNS: string
#
def sec2time(seconds):
    if seconds > (60*60):
        try:
            time_string = strftime("%H:%M:%S", gmtime(int(seconds)))
        except ValueError:
            return "00:00:00"
    else:
        try:
            time_string = strftime("%M:%S", gmtime(int(seconds)))
        except ValueError:
            return "00:00"
    return time_string

###################################################################
#
# Convert numeric genre-id to genre names
# RETURNS: string
#
def id2genre(id):
    genres = [
    "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", 
    "Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies", "Other", 
    "Pop", "R&amp;B", "Rap", "Reggae", "Rock", "Techno", "Industrial", 
    "Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack", 
    "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion", 
    "Trance", "Classical", "Instrumental", "Acid", "House", "Game", 
    "Sound Clip", "Gospel", "Noise", "Alt. Rock", "Bass", "Soul", 
    "Punk", "Space", "Meditative", "Instrum. Pop", "Instrum. Rock", 
    "Ethnic", "Gothic", "Darkwave", "Techno-Indust.", "Electronic", 
    "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy", 
    "Cult", "Gangsta", "Top 40", "Christian Rap", "Pop/Funk", "Jungle", 
    "Native American", "Cabaret", "New Wave", "Psychadelic", "Rave", 
    "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", 
    "Polka", "Retro", "Musical", "Rock &amp; Roll", "Hard Rock", "Folk", 
    "Folk/Rock", "National Folk", "Swing", "Fusion", "Bebob", "Latin", 
    "Revival", "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock", 
    "Progress. Rock", "Psychadel. Rock", "Symphonic Rock", "Slow Rock", 
    "Big Band", "Chorus", "Easy Listening", "Acoustic", "Humour", 
    "Speech", "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony", 
    "Booty Bass", "Primus", "Porn Groove", "Satire", "Slow Jam", 
    "Club", "Tango", "Samba", "Folklore", "Ballad", "Power Ballad", 
    "Rhythmic Soul", "Freestyle", "Duet", "Punk Rock", "Drum Solo", 
    "A Capella", "Euro-House", "Dance Hall", "Goa", "Drum &amp; Bass", 
    "Club-House", "Hardcore", "Terror", "Indie", "BritPop", "Negerpunk", 
    "Polsk Punk", "Beat", "Christian Gangsta Rap", "Heavy Metal", 
    "Black Metal", "Crossover", "Contemporary Christian", "Christian Rock",
    "Merengue", "Salsa", "Thrash Metal", "Anime", "Jpop", "Synthpop" 
    ]
    if id == None:
        return "n/A"
    return genres[int(id)]

###################################################################
#
# gettext - _()
#
def translation_gettext():
    path = "locale"
    if not os.path.exists(path):
        path = os.path.join(install_dir(), "locale")
    try:
        trans = gettext.translation('pyjama', path)
    except IOError:
        trans = gettext.translation('pyjama', path, ["en"])
        print "No language file for you found - using english language file"
    trans.install()

###################################################################
#
# Checks if there is available a new version
# RETURNS: 0 = no new version, 1 = new version, 2 = force new version
#
def check_version():
    VERSION_URL

###################################################################
#
# Checks if some necessary modules can be found
# RETURNS: int
#    
def check_modules():
    err = 0
    nf1, nf2 = False, False

    try:
        import pygtk
    except ImportError:
        err += 1
        print ("Module '%s' not found") % "pygtk"

    try:
        import gtk
    except ImportError:
        err += 1
        print ("Module '%s' not found") % "gtk"
        
    try:
        import gtk.glade
    except ImportError:
        err += 1
        print ("Module '%s' not found") % "gtk.glade"
        
    try:
        import simplejson
    except ImportError:
        err += 1
        print ("Module '%s' not found") % "simplejson"

    try:
        import gst
    except ImportError:
        err += 1
        print ("Module '%s' not found") % "gst (gstreamer)"
        
    try:
        import xml.sax
    except ImportError:
        err += 1
        print ("Module '%s' not found") % "xml.sax"

    try:
        import sqlite3
    except ImportError:
        nf1 = True
        
    try:
        import pysqlite2
    except ImportError:
        nf2 = True
        if nf1 and nf2:
            err += 1
            print ("You need to have sqlite3 or pysqlite2 or libsqlite3-0 installed")

    if err == 0:
        print ("All modules found")
    else:
        print ("%s module(s) missing") % err

    return err

        
