#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2009 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import gtk
import pickle
import md5
import os

import functions
import clEntry

import clBookmarkDialog

class main():
    def __init__(self, pyjama):
        self.bookmarks = []
        self.pyjama = pyjama
        self.bookmarks_dir = os.path.join(self.pyjama.home, "bookmarks")

        # create bookmarks directory in ~/.pyjama
        home = self.pyjama.home
        if os.path.exists(os.path.join(home,  "bookmarks")) == False:
            try:
                os.makedirs(os.path.join(home, "bookmarks"))
                print "Created %s" % os.path.join(home, "bookmarks")
            except:
                self.pyjama.Events.raise_event("error", _("Could not create folder '%s' - most probably\npyjama does not have the right privilegs to create that folder" % os.path.join(home, "bookmarks")))
                return

        # hide from configtool
        bl = self.pyjama.settings.get_value("CONFIGTOOL", "blacklist", "PATHs URLs")
        if not "Bookmarks" in bl:
            bl += " Bookmarks"
            self.pyjama.settings.set_value("CONFIGTOOL", "blacklist", bl)

        self.pyjama.Events.connect_event("alldone", self.ev_alldone)

#        gladefile = os.path.join(functions.install_dir(), "plugins" , "bookmarks", "bookmarks.glade")
#        try:
#            self.wTree = gtk.glade.XML(gladefile)  
#            self.dialog = self.wTree.get_widget("dialog1")
#            self.ok = self.wTree.get_widget("button1")
#            self.ok.connect("clicked", self.ev_ok_clicked)
#        except Exception, inst:
#            self.pyjama.Events.raise_event("error", inst, "Error loading glade file %s" % gladefile)
#            self.dialog = None

    ######################################################################
    #                                                                    #
    #                           Functions                                #
    #                                                                    #
    ######################################################################

    def edit_bookmarks(self, widget):
        self.pyjama.Events.raise_event("error", "Editing Bookmarks wasn't implemented, yet")
        return
        # load dialog
        self.dialog = clBookmarkDialog.Dialog(self, self.bookmarks)
        response = self.dialog.run()
        if response == 0:
            self.dialog.destroy()
            print "ok geklickt"


    def add_bookmark(self, widget):

        data = self.pyjama.historyCurrent
        content = pickle.dumps(data, 2)
        md5_hash = md5.new(content).hexdigest()

        # ask for name
        result = clEntry.input_box(title=_('Bookmark'),
            message=_('Please enter a Name for this bookmark'),
            default_text=_("Bookmark"))
        if result is None:
            return
        name = str(result)
        
        # Writing file to disc
        try:
            fh = open(os.path.join(self.bookmarks_dir, md5_hash), "w")
        except Exception, inst:
            self.pyjama.Events.raise_event("error", inst, "Could not write Bookmark")
            return
        fh.write(content)
        fh.close()
        try:
            os.chmod(os.path.join(self.bookmarks_dir, md5_hash), 0777)
        except Exception, inst:
            self.pyjama.Events.raise_event("error", inst, "Error chmodding bookmark file")

        # Append to bookmarks and write to config
        self.pyjama.settings.set_value("Bookmarks", md5_hash, name)

        # Append menu entry
        menu = self.pyjama.window.menubar
        entry = menu.append_entry(menu.get_rootmenu("Bookmarks"), name, md5_hash)
        entry.connect("activate", self.ev_load_bookmark, md5_hash)

        self.bookmarks.append({'name':name, 'hash':md5_hash})

    def read_bookmark_folder(self):
        if self.pyjama.settings.config.has_section("Bookmarks"):
            for md5_hash, name in self.pyjama.settings.config.items("Bookmarks"):
                # Append menu entry
                menu = self.pyjama.window.menubar
                entry = menu.append_entry(menu.get_rootmenu("Bookmarks"), name, md5_hash)
                entry.connect("activate", self.ev_load_bookmark, md5_hash)

                self.bookmarks.append({'name':name, 'hash':md5_hash})
        
        


    ######################################################################
    #                                                                    #
    #                                Events                              #
    #                                                                    #
    ######################################################################

    def ev_ok_clicked(self, widget):
        self.dialog.destroy()

    def ev_alldone(self):
        # Create Bookmarks menu
        menu = self.pyjama.window.menubar
        bookmarks = menu.insert_root_before("Extras", "Bookmarks", _("Bookmarks"))

        sub = ["Add Bookmark", "Edit Bookmarks", "---"]  
        self.bookmark_menu = menu.create_submenu(rootmenu=bookmarks, submenu=sub)

        add_bookmark = menu.get_submenu("Add Bookmark")
        add_bookmark.connect("activate", self.add_bookmark)

        edit_bookmarks = menu.get_submenu("Edit Bookmarks")
        edit_bookmarks.connect("activate", self.edit_bookmarks)

        # Create accel group
        self.accel_group = gtk.AccelGroup()
        add_bookmark.add_accelerator("activate", self.accel_group, ord("d"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
        self.pyjama.window.add_accel_group(self.accel_group)

        self.read_bookmark_folder()

    def ev_load_bookmark(self, widget, md5_hash):
        # Read file
        try: 
            fh = open(os.path.join(self.bookmarks_dir, md5_hash), "r")
        except:
            self.pyjama.Events.raise_event("error", _("Could not read bookmark file"))
            return
        content = fh.read()
        fh.close()

        # unserialise and show
        data = pickle.loads(content)
        self.pyjama.layouts.show_layout(data['layout'], data['data1'], data['data2'], data['data3'], data['data4'])
