#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import os
import gtk
import gtk.glade

import functions

try:
    import gtkmozembed # ??? python2.5-gnome2-extras
except:
    print "AN ERROR OCCURED:"
    print "-------------------------------------------------"
    print "Probably you need to install python-gnome-extras!"
    raise ImportError, 'gtkmozembed'      

class main():
    def __init__(self, pyjama):
        self.pyjama = pyjama 
        self.home = functions.preparedirs()
        profdir = os.path.join(self.home, "mozplug")
        profile = "default"
        prefsPath = os.path.join(profdir,profile,'prefs.js') 

        self.pyjama.Events.connect_event("alldone", self.ev_alldone)
        self.pyjama.Events.connect_event("showing_artist_page", self.ev_artistpage)
        self.pyjama.Events.connect_event("showing_album_page", self.ev_albumpage)

        prefsContent = """
user_pref("browser.link.open_newwindow", 1);
user_pref("browser.link.open_newwindow.restriction", 0);
user_pref("browser.link.open_external", 1);
user_pref("capability.policy.default.Window.open", "noAccess");
user_pref("security.warn_entering_secure", false); 
user_pref("security.warn_entering_weak", false); 
user_pref("security.warn_viewing_mixed", false); 
user_pref("security.warn_leaving_secure", false); 
user_pref("security.warn_submit_insecure", false); 
user_pref("security.warn_entering_secure.show_once", false); 
user_pref("security.warn_entering_weak.show_once", false); 
user_pref("security.warn_viewing_mixed.show_once", false); 
user_pref("security.warn_leaving_secure.show_once", false); 
user_pref("security.warn_submit_insecure.show_once", false); 
user_pref("security.enable_java", true); 
user_pref("browser.xul.error_pages.enabled", false); 
user_pref("general.useragent.vendor", "%s"); 
user_pref("general.useragent.vendorSub", "%s"); 
user_pref("general.useragent.vendorComment", "%s"); 
        """ % ("Pyjama", self.pyjama.version, "Python Jamendo Audioplayer")

        if os.path.exists(os.path.join(self.home, "mozplug", "default")) == False:
            os.makedirs(os.path.join(self.home, "mozplug", "default"))
            functions.preparedirs(set_privilegs=True)
            f = open(prefsPath, "wt") 
            f.write(prefsContent) 
            f.close() 

        self.mozillaWidget = gtkmozembed.MozEmbed()
        gtkmozembed.set_profile_path(profdir, profile)
        self.mozillaWidget.connect("location", self.ev_location_changed)
        self.mozillaWidget.connect("link_message", self.ev_link_message)
        self.mozillaWidget.connect("new-window", self.ev_new_window)
        self.cururl = "http://www.jamendo.com"
        self.mozillaWidget.load_url(self.cururl)
        self.settingurl = False

    def new_browser(self, chromemask):
        import functions
        curdir = os.path.abspath(os.path.curdir)
        curdir = os.path.join(curdir, "plugins")
        self.gladefile = os.path.join(functions.install_dir(), "plugins" , "mozplug", "mozplug.glade")
        try:
            self.wTree = gtk.glade.XML(self.gladefile)  
        except RuntimeError:
            print "Error loading glade file"
            print "tried to load it from: ", self.gladefile
        window = self.wTree.get_widget("window1")
        fixed = self.wTree.get_widget("fixed1")
        mozillaWidget = gtkmozembed.MozEmbed()
        fixed.put(mozillaWidget, 0,0)
        return  window


    def ev_new_window(self, Widget, retval, chromemask):
        self.pyjama.Events.raise_event("error",  "Mozplug is not able to display this page,\n since this would require to open a new window.\n Mozplug does not support opening new windows.")
#        return 
        retval = self.new_browser(chromemask)
        return  retval

    def ev_location_changed(self, Widget):
        url = Widget.get_location()
        url = url.decode('utf-8').strip()
        if url[-8:] == self.cururl[-8:]: url = self.cururl
        if url != self.cururl:
            print url
            print self.cururl
            self.settingurl = True
            self.cururl = url
            arr = url.split("/")
            if len(arr) >= 2:
                if arr[-2] == "album":
                    id = arr[-1].replace("#", "")
                    print "Album: ", id
                    albumdetails = self.pyjama.jamendo.albuminfos(id)
                    self.pyjama.layouts.show_layout("album", albumdetails, who_called = "ev_location_changed-mozplug")
                elif arr[-2] == "artist":
                    id = arr[-1].replace("#", "")
                    for char in [".", "/", ".", "\\", ","]:
                        id = id.replace(char, " ")
                    sql = """
                        SELECT
                            artists.id
                        FROM
                            artists
                        WHERE
                            artists.name='%s'
                        """ % str(id)
                    ret = self.pyjama.db.query(sql)
                    if ret:
                        artistid = ret[0][0]
                    else:
                        return None
                    print "Artist:", artistid
                    artist = self.pyjama.db.artistinfos(artistid)
                    self.pyjama.layouts.show_layout("artist", artist, who_called = "ev_location_changed-mozplug")

    def ev_link_message(self, Widget):
        pass
#        print Widget.get_link_message()

    def ev_progress(self,  mozWidget, cur, max):
        print max, ": ", cur

    def ev_artistpage(self, artistinfos): 
        self.settingurl = False  
        url = artistinfos[0]['artist_url']
        self.goto_url( url )

    def ev_albumpage(self, albuminfos):
        settings = self.pyjama.settings
        url = settings.get_value("URLs", 'album_url').replace("URL", "%s" % albuminfos['id'])
        self.settingurl = False  
        self.goto_url( url )
        
    def ev_alldone(self):
        register =  self.pyjama.settings.get_value("MOZPLUG", 'REGISTER_AS_BROWSER', -1)
        if register == -1:
            self.pyjama.settings.set_value("MOZPLUG", "REGISTER_AS_BROWSER", "True")
            register = True
        if register == True:
            # register mozplug as pyjama's default browser!
            self.pyjama.browser.set_browser(self.goto_url)        

        self.pyjama.window.vpaned2.pack2(self.mozillaWidget, True, True)
        self.mozillaWidget.show()

    def goto_url(self, url):
        if not self.settingurl:
            self.cururl = url
            self.mozillaWidget.load_url(url)
        else:
            self.settingurl = True

#    def goto_url2(self, url):
#        if not self.settingurl:
#            self.cururl = url
#            self.mozillaWidget.load_url(url)
#        else:
#            self.settingurl = True
        
        
