#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2009 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import os, gtk

import functions
from clWidgets import ImageButton, StockButton
import clStarLayout

class main():
    def __init__(self, pyjama):
        self.pyjama = pyjama

        # Connect to AlbumInfo Widget creation
        self.pyjama.Events.connect_event("albuminfo_created", self.ev_albuminfo_created)
        self.pyjama.Events.connect_event("showing_album_page", self.ev_showing_album_page)
        self.pyjama.Events.connect_event("alldone", self.ev_alldone)


    ######################################################################
    #                                                                    #
    #                                Events                              #
    #                                                                    #
    ######################################################################

    # Called when album-view was created
    def ev_alldone(self):
        # add star buttont to main toolbar
        self.bStar = gtk.ToolButton()
        self.bStar.set_label(_("Stared"))
        self.bStar.set_tooltip_text(_("Show stared Albums"))
        pos = self.pyjama.window.toolbar.get_item_index(self.pyjama.window.toolbar.bFullScreen)
        self.pyjama.window.toolbar.insert(self.bStar, pos)
        self.pyjama.window.toolbar.set_image(self.bStar, os.path.join(functions.install_dir(), "images", "star.png"))
        self.bStar.connect("clicked", self.ev_show_star_layout)
        self.bStar.show()

        # add star button to album view
        self.ibStarAlbum = ImageButton(os.path.join(functions.install_dir(), "images", "star.png"), 26, 26)#StockButton(gtk.STOCK_STAR, gtk.ICON_SIZE_LARGE_TOOLBAR)
        self.pyjama.layouts.toolbars['album'].pack_start(self.ibStarAlbum, False, True, 2)
        self.ibStarAlbum.set_tooltip_text(_("Star this album"))
        self.ibStarAlbum.set_size_request(40,40)
        self.ibStarAlbum.show()
        self.ibStarAlbum.connect("clicked", self.on_StarAlbum, "tag")

        self.pyjama.layouts.register_layout("star", clStarLayout.StarLayout(self.pyjama))

        # Append menu entry
        menu = self.pyjama.window.menubar
        entry = menu.append_entry(menu.get_rootmenu("Extras"), "Star", "Star")
        entry.connect("activate", self.ev_show_star_layout)

        self.accel_group = gtk.AccelGroup()
        entry.add_accelerator("activate", self.accel_group, gtk.keysyms.F7, 0, gtk.ACCEL_VISIBLE)
        self.pyjama.window.add_accel_group(self.accel_group)

    def ev_show_star_layout(self, widget):
        self.pyjama.layouts.show_layout("star", 10, "ratingweek", 1, "all")


    def ev_albuminfo_created(self, widget):
        albumid = str(widget.album['album_id'])
        stared = self.pyjama.settings.get_value("STAR PLUGIN", "STARED ALBUMS", "")
        if albumid in stared:
            # add remove button to widget
            self.bControlStar = StockButton(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
            self.bControlStar.set_tooltip_text(_("Un-star this album"))
            widget.vbControl.pack_start(self.bControlStar, False, True, 10)
            self.bControlStar.connect("clicked", self.on_StarAlbum, albumid)
            self.bControlStar.show()
        else:
            # Add Star to widget
            self.bControlStar = ImageButton(os.path.join(functions.install_dir(), "images", "star.png"), 16, 16)
            self.bControlStar.set_tooltip_text(_("Star this album"))
            widget.vbControl.pack_start(self.bControlStar, False, True, 10)
            self.bControlStar.connect("clicked", self.on_StarAlbum, albumid)
            self.bControlStar.show()

    def ev_showing_album_page(self, albuminfos):
        albumid = str(albuminfos['id'])
        stared = self.pyjama.settings.get_value("STAR PLUGIN", "STARED ALBUMS", "")
        if albumid in stared:
            # show remove button
            self.ibStarAlbum.set_tooltip_text(_("Un-star this album"))
            self.ibStarAlbum.setstock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_LARGE_TOOLBAR)
        else:
            # show add button
            self.ibStarAlbum.set_tooltip_text(_("Star this album"))
            self.ibStarAlbum.setimage(os.path.join(functions.install_dir(), "images", "star.png"), 26, 26)
        # Set id as tag to read out later
        self.ibStarAlbum.tag = albumid

    # Add Star to Contextmenu

#    def on_ibStarAlbum_clicked(self, widget, albumid):
#        print albumid
    
    def on_StarAlbum(self, widget, albumid=None):
        if albumid == "tag":
            albumid = self.ibStarAlbum.tag
        stared = self.pyjama.settings.get_value("STAR PLUGIN", "STARED ALBUMS", "")
        if not str(albumid) in stared:
            # add album to stares
            stared = "%s+%s" % (stared, albumid)
            self.pyjama.settings.set_value("STAR PLUGIN", "STARED ALBUMS", stared)
        else:
            # remove album
            stared = stared.replace("+%s" % str(albumid), "")
            self.pyjama.settings.set_value("STAR PLUGIN", "STARED ALBUMS", stared)
        self.pyjama.reload_current_page()



